@@ -1,4 +1,4 @@
-// Copyright (c) 2014-2016 Josh Blum
+// Copyright (c) 2014-2017 Josh Blum
 // SPDX-License-Identifier: GPL-3.0
 
 /***********************************************************************
@@ -37,7 +37,7 @@ public:
         _type(args.at("type")),
         _isNetworkDevice(args.count("addr") != 0)
     {
-        if (args.count("rx_subdev") != 0) _dev->set_tx_subdev_spec(args.at("rx_subdev"));
+        if (args.count("rx_subdev") != 0) _dev->set_rx_subdev_spec(args.at("rx_subdev"));
         if (args.count("tx_subdev") != 0) _dev->set_tx_subdev_spec(args.at("tx_subdev"));
     }
 
@@ -539,6 +539,13 @@ public:
 
     double getFrequency(const int dir, const size_t channel, const std::string &name) const
     {
+        //we have never tuned before, return the overall freq for RF, assume 0.0 for all else
+        if (_trCache.count(dir) == 0 or _trCache.at(dir).count(channel) == 0)
+        {
+            if (name == "RF") return this->getFrequency(dir, channel);
+            else return 0.0;
+        }
+
         const uhd::tune_result_t tr = _trCache.at(dir).at(channel);
         if (name == "RF") return tr.actual_rf_freq;
         if (name == "BB") return tr.actual_dsp_freq;
@@ -602,6 +609,13 @@ public:
         return SoapySDR::Device::listSampleRates(dir, channel);
     }
 
+    SoapySDR::RangeList getSampleRateRange(const int dir, const size_t channel) const
+    {
+        if (dir == SOAPY_SDR_TX) return metaRangeToRangeList(_dev->get_tx_rates(channel));
+        if (dir == SOAPY_SDR_RX) return metaRangeToRangeList(_dev->get_rx_rates(channel));
+        return SoapySDR::Device::getSampleRateRange(dir, channel);
+    }
+
     void setBandwidth(const int dir, const size_t channel, const double bw)
     {
         if (dir == SOAPY_SDR_TX) return _dev->set_tx_bandwidth(bw, channel);
@@ -622,6 +636,13 @@ public:
         return SoapySDR::Device::listBandwidths(dir, channel);
     }
 
+    SoapySDR::RangeList getBandwidthRange(const int dir, const size_t channel) const
+    {
+        if (dir == SOAPY_SDR_TX) return metaRangeToRangeList(_dev->get_tx_bandwidth_range(channel));
+        if (dir == SOAPY_SDR_RX) return metaRangeToRangeList(_dev->get_tx_bandwidth_range(channel));
+        return SoapySDR::Device::getBandwidthRange(dir, channel);
+    }
+
     /*******************************************************************
      * Clocking support
      ******************************************************************/
@@ -839,6 +860,17 @@ std::vector<SoapySDR::Kwargs> find_uhd(const SoapySDR::Kwargs &args_)
     for (size_t i = 0; i < addrs.size(); i++)
     {
         SoapySDR::Kwargs result(dictToKwargs(addrs[i]));
+
+        //create displayable label if not present
+        if (result.count("label") == 0)
+        {
+            if (result.count("product") != 0)
+                result["label"] = result.at("product");
+
+            if (result.count("serial") != 0)
+                result["label"] += " " + result.at("serial");
+        }
+
         result.erase(SOAPY_UHD_NO_DEEPER);
         results.push_back(result);
     }
